package de.fzj.unicore.uas.impl.dsms;

import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.impl.UASWSResourceImpl;
import de.fzj.unicore.uas.impl.sms.StorageManagementHomeImpl;
import de.fzj.unicore.uas.impl.sms.StorageManagementHomeImpl.StorageTypes;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.KernelInjectable;
import de.fzj.unicore.wsrflite.exceptions.ResourceNotCreatedException;
import de.fzj.unicore.wsrflite.exceptions.ResourceUnknownException;
import de.fzj.unicore.wsrflite.xmlbeans.impl.WSResourceImpl;

public class CreateDSMSOnStartup implements Runnable, KernelInjectable {
	private static final Logger logger = LogUtil.getLogger(LogUtil.UNICORE, CreateDSMSOnStartup.class);

	private static final String DSMS_RES_ID_PREFIX = "dsms_";

	public static final String DEFAULT_RESOURCE_ID = "default_dsms";

	public static final String DEFAULT_CONF_ID = "DEFAULT";

	private static final String MULTIPLE_DSMS = "multiple_dsms";

	private Kernel kernel;

	public void setKernel(Kernel kernel) {
		this.kernel = kernel;
	}

	public void run() {
		Home smsHome = kernel.getHome(UAS.SMS);
		if (smsHome == null) {
			logger.info("No dSMS configured for this site.");
			return;
		}

		if (kernel.getBooleanProperty(DSMSImpl.DSMS_CONF_PREFIX + "." + MULTIPLE_DSMS, false)) {
			Collection<String> confIds = getConfIds();
			for (String confId : confIds) {
				createIfNotExists(smsHome, getResId(confId), confId);
			}
		} else {
			createIfNotExists(smsHome, DEFAULT_RESOURCE_ID, null);
		}
	}

	private void createIfNotExists(Home smsHome, String resId, String confId) {
		logger.debug("Creating " + resId + " from " + DSMSImpl.DSMS_CONF_PREFIX + "." + confId);
		try {
			try {
				if (smsHome.get(resId) != null) {
					logger.info(resId + " already exists");
					return;
				}
			} catch (ResourceUnknownException e) {
			}
			doCreateSMS(smsHome, resId, confId);
		} catch (Exception e) {
			logger.error("Could not add " + resId, e);
		}
	}

	private void doCreateSMS(Home smsHome, String resId, String confId) throws ResourceNotCreatedException {
		Map<String, Object> imap = new HashMap<String, Object>();
		imap.put(WSResourceImpl.INIT_UNIQUE_ID, resId);
		imap.put(DSMSImpl.DSMS_CONF_ID, confId);

		Calendar c = new GregorianCalendar();
		c.add(Calendar.MONTH, 72);
		imap.put(WSResourceImpl.INIT_INITIAL_TERMINATION_TIME, c);
		imap.put(UASWSResourceImpl.INITPARAM_PUBLISH_TO_REGISTRY, true);
		imap.put(StorageManagementHomeImpl.INIT_STORAGETYPE, StorageTypes.DSMS);
		smsHome.createWSRFServiceInstance(imap);
		logger.info("Added dSMS resource " + resId + ".");
	}

	private Collection<String> getConfIds() {
		Set<String> confIds = new HashSet<String>();
		String keyPrefix = DSMSImpl.DSMS_CONF_PREFIX + ".";
		for (Object o : kernel.getProperties().keySet()) {
			if (!(o instanceof String)) {
				continue;
			}
			String key = (String) o;
			if (key.startsWith(keyPrefix)) {
				String id = key.substring(keyPrefix.length());
				int dotIndex = id.indexOf('.');
				if(dotIndex != -1) {
					id = id.substring(0, dotIndex);
				}
				if (MULTIPLE_DSMS.equals(id)) {
					continue;
				}
				if (kernel.getProperty(keyPrefix + id + "." + DSMSImpl.DSMS_NAME) != null) {
					confIds.add(id);
				} else {
					logger.error(keyPrefix + id + " doesn't have mandatory name property");
				}
			}
		}
		return confIds;
	}

	private String getResId(String confId) {
		if (DEFAULT_CONF_ID.equals(confId)) {
			return DEFAULT_RESOURCE_ID;
		} else {
			return DSMS_RES_ID_PREFIX + confId.toLowerCase();
		}
	}
}