/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved. See LICENCE file for licensing
 * information.
 * 
 * Author: T. Rękawek <newton@mat.uni.torun.pl>
 */

package de.fzj.unicore.uas.impl.dsms;

import java.util.Map;

import org.apache.log4j.Logger;
import org.unigrids.x2006.x04.services.sms.ChangePermissionsDocument;
import org.unigrids.x2006.x04.services.sms.ChangePermissionsResponseDocument;
import org.unigrids.x2006.x04.services.sms.CopyDocument;
import org.unigrids.x2006.x04.services.sms.CopyResponseDocument;
import org.unigrids.x2006.x04.services.sms.CreateDirectoryDocument;
import org.unigrids.x2006.x04.services.sms.CreateDirectoryResponseDocument;
import org.unigrids.x2006.x04.services.sms.DeleteDocument;
import org.unigrids.x2006.x04.services.sms.DeleteResponseDocument;
import org.unigrids.x2006.x04.services.sms.ExportFileDocument;
import org.unigrids.x2006.x04.services.sms.ExportFileResponseDocument;
import org.unigrids.x2006.x04.services.sms.FindDocument;
import org.unigrids.x2006.x04.services.sms.FindResponseDocument;
import org.unigrids.x2006.x04.services.sms.ImportFileDocument;
import org.unigrids.x2006.x04.services.sms.ImportFileResponseDocument;
import org.unigrids.x2006.x04.services.sms.ListDirectoryDocument;
import org.unigrids.x2006.x04.services.sms.ListDirectoryResponseDocument;
import org.unigrids.x2006.x04.services.sms.ListPropertiesDocument;
import org.unigrids.x2006.x04.services.sms.ListPropertiesResponseDocument;
import org.unigrids.x2006.x04.services.sms.ReceiveFileDocument;
import org.unigrids.x2006.x04.services.sms.ReceiveFileResponseDocument;
import org.unigrids.x2006.x04.services.sms.RenameDocument;
import org.unigrids.x2006.x04.services.sms.RenameResponseDocument;
import org.unigrids.x2006.x04.services.sms.SendFileDocument;
import org.unigrids.x2006.x04.services.sms.SendFileResponseDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import pl.edu.icm.dsms.catalogue.CatalogueClient;
import pl.edu.icm.dsms.catalogue.CatalogueClient.LdasAndEprs;
import pl.edu.icm.dsms.catalogue.CatalogueUtils;
import pl.edu.icm.dsms.catalogue.model.Lda;
import pl.edu.icm.x2010.x10.services.catalogue.SharedStoragePropertiesType;
import pl.edu.icm.x2010.x10.services.catalogue.CreateSmsResponseDocument.CreateSmsResponse;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.impl.dsms.operation.ChangePermissions;
import de.fzj.unicore.uas.impl.dsms.operation.Copy;
import de.fzj.unicore.uas.impl.dsms.operation.CreateDirectory;
import de.fzj.unicore.uas.impl.dsms.operation.Delete;
import de.fzj.unicore.uas.impl.dsms.operation.ExportFile;
import de.fzj.unicore.uas.impl.dsms.operation.Find;
import de.fzj.unicore.uas.impl.dsms.operation.ImportFile;
import de.fzj.unicore.uas.impl.dsms.operation.ListDirectory;
import de.fzj.unicore.uas.impl.dsms.operation.ListProperties;
import de.fzj.unicore.uas.impl.dsms.operation.ReceiveFile;
import de.fzj.unicore.uas.impl.dsms.operation.Rename;
import de.fzj.unicore.uas.impl.dsms.operation.SendFile;
import de.fzj.unicore.uas.impl.dsms.operation.helper.CatalogueClientUtils;
import de.fzj.unicore.uas.impl.dsms.operation.helper.ClientHelper;
import de.fzj.unicore.uas.impl.sms.SMSBaseImpl;
import de.fzj.unicore.uas.metadata.MetadataManager;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.messaging.ResourceDeletedMessage;
import de.fzj.unicore.wsrflite.persistence.Persist;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.xmlbeans.BaseFault;
import eu.unicore.security.util.Log;

/**
 * Distributed Storage Management Service
 * 
 * @author newton
 * 
 */
public class DSMSImpl extends SMSBaseImpl {
	private static final Logger logger = Log.getLogger(LogUtil.SERVICES, DSMSImpl.class);

	// default protocol for copy operations
	private final static String PROTOCOL = "RBYTEIO";

	static final String DSMS_CONF_PREFIX = "uas.dsms";

	private static final String ENABLE_SYNC_1 = "sync1";

	private static final String ENABLE_SYNC_2 = "sync2";

	private static final String CATALOGUE_URL = "catalogue";

	static final String DSMS_NAME = "name";

	public static final String DSMS_CONF_ID = "dSmsConfId";

	private static DSMSCatalogueCache CACHE;

	static {
		initCache();
	}

	@Persist
	private String dSmsId;

	@Persist
	private String dSmsConfId;

	@Persist
	private String dSmsRoot;

	@Persist
	private EndpointReferenceType catalogueEpr;

	@Override
	public void initialise(String name, Map<String, Object> initobjs) throws Exception {
		super.initialise(name, initobjs);
		if (initobjs.containsKey(DSMS_CONF_ID)) {
			dSmsConfId = (String) initobjs.get(DSMS_CONF_ID);
		}
		workdir = getStorageRoot();

		catalogueEpr = CatalogueClientUtils.createCatalogueEpr(getKernel(),
				getConfProperty(CATALOGUE_URL, null));
		if (catalogueEpr == null) {
			throw new Exception("Can't get catalogue");
		}
		properties.put(RPFileSystem, new DSMSFileSystemRP(this, getConfProperty(DSMS_NAME, "DSMS"), catalogueEpr));
		CatalogueClient cat = CatalogueClientUtils.getCatalogueClient(getKernel().getSecurityProperties(),
				catalogueEpr);
		if (storageFactoryID == null) {
			SharedStoragePropertiesType catProps = cat.getSharedStorageProperties();
			this.dSmsRoot = catProps.getGlobalDsmsRoot();
			this.dSmsId = catProps.getGlobalDsmsId();
		} else {
			CreateSmsResponse response = cat.createSms(getUniqueID(), getOwner());
			this.dSmsRoot = response.getDSmsRoot();
			this.dSmsId = response.getDSmsId();
		}
		logger.info("Initialised dSMS " + (dSmsId == null ? "" : dSmsId));
	}

	/**
	 * get the {@link MetadataManager}<br/>
	 * This implementation will return a dSMS metadata manager
	 * 
	 * @return {@link MetadataManager} or <code>null</code> if disabled
	 */
	protected MetadataManager getMetadataManager() throws Exception {
		if (disableMetadata) {
			return null;
		}
		return new DSMSMetadataManager(getContext());
	}

	@Override
	public void destroy() {
		if (storageFactoryID != null) {
			try {
				ResourceDeletedMessage m = new ResourceDeletedMessage("deleted:" + getUniqueID());
				m.setDeletedResource(getUniqueID());
				m.setServiceName(getServiceName());
				getKernel().getMessaging().getChannel(storageFactoryID).publish(m);
			} catch (Exception e) {
				logger.error("Could not send message to StorageFactory", e);
			}
		}
		try {
			ClientHelper clientHelper = new ClientHelper(getContext());
			if (cleanupOnDestroy != null && cleanupOnDestroy) {
				LdasAndEprs lae = clientHelper.getCatalogueClient().removeSms(dSmsId);
				for (EndpointReferenceType epr : lae.getEprs()) {
					ISecurityProperties sec = CatalogueUtils.getTrustDelegation(kernel, getClient());
					StorageClient storageClient = new StorageClient(epr, sec);
					storageClient.destroy();
				}
				for (Lda lda : lae.getLdas()) {
					StorageClient storageClient = clientHelper.getStorageClient(lda);
					storageClient.delete(lda.getPhysicalName());
				}
			}
		} catch (Exception e) {
			logger.error("Could not destroy dSMS", e);
		}
		super.destroy();
		logger.info("Destroyed dSMS " + (dSmsId == null ? "" : dSmsId));
	}

	public ChangePermissionsResponseDocument ChangePermissions(ChangePermissionsDocument in) throws BaseFault {
		return (ChangePermissionsResponseDocument) new ChangePermissions(getContext()).perform(in);
	}

	public CopyResponseDocument Copy(CopyDocument in) throws BaseFault {
		return (CopyResponseDocument) new Copy(getContext()).perform(in);
	}

	public CreateDirectoryResponseDocument CreateDirectory(CreateDirectoryDocument in) throws BaseFault {
		return (CreateDirectoryResponseDocument) new CreateDirectory(getContext()).perform(in);
	}

	public DeleteResponseDocument Delete(DeleteDocument in) throws BaseFault {
		return (DeleteResponseDocument) new Delete(getContext()).perform(in);
	}

	public ExportFileResponseDocument ExportFile(ExportFileDocument in) throws BaseFault {
		return (ExportFileResponseDocument) new ExportFile(getContext()).perform(in);
	}

	public FindResponseDocument Find(FindDocument in) throws BaseFault {
		return (FindResponseDocument) new Find(getContext()).perform(in);
	}

	public ImportFileResponseDocument ImportFile(ImportFileDocument in) throws BaseFault {
		return (ImportFileResponseDocument) new ImportFile(getContext()).perform(in);
	}

	public ListDirectoryResponseDocument ListDirectory(ListDirectoryDocument in) throws BaseFault {
		return (ListDirectoryResponseDocument) new ListDirectory(getContext()).perform(in);
	}

	public ListPropertiesResponseDocument ListProperties(ListPropertiesDocument in) throws BaseFault {
		return (ListPropertiesResponseDocument) new ListProperties(getContext()).perform(in);
	}

	public ReceiveFileResponseDocument ReceiveFile(ReceiveFileDocument in) throws BaseFault {
		return (ReceiveFileResponseDocument) new ReceiveFile(getContext()).perform(in);
	}

	public RenameResponseDocument Rename(RenameDocument in) throws BaseFault {
		return (RenameResponseDocument) new Rename(getContext()).perform(in);
	}

	public SendFileResponseDocument SendFile(SendFileDocument in) throws BaseFault {
		return (SendFileResponseDocument) new SendFile(getContext()).perform(in);
	}

	@Override
	protected String getStorageRoot() {
		return "/tmp";
	}

	@Override
	protected void setupMetadataService() throws Exception {
		logger.info("Setting up metadata");
		logger.info("Disable metadata: " + disableMetadata);
		super.setupMetadataService();
	}

	private String getConfProperty(String name, String defaultValue) {
		StringBuilder b = new StringBuilder(DSMS_CONF_PREFIX);
		if (dSmsConfId != null && !dSmsConfId.isEmpty()) {
			b.append('.').append(dSmsConfId);
		}
		b.append('.').append(name);
		return kernel.getProperty(b.toString(), defaultValue);
	}

	public static void initCache() {
		CACHE = new DSMSCatalogueCache();
	}

	private OperationContext getContext() throws BaseFault {
		OperationContext context = new OperationContext();
		context.setCache(CACHE);
		context.setClient(getClient());
		context.setCreateRoot(this.storageFactoryID != null);
		context.setdSmsId(dSmsId);
		context.setdSmsRoot(dSmsRoot);
		context.setKernel(getKernel());
		context.setProtocol(PROTOCOL);
		context.setSync1(Boolean.parseBoolean(getConfProperty(ENABLE_SYNC_1, "false")));
		context.setSync2(Boolean.parseBoolean(getConfProperty(ENABLE_SYNC_2, "false")));
		context.setCatalogueEpr(catalogueEpr);
		return context;
	}
}