package de.fzj.unicore.uas.security;


import org.apache.log4j.Logger;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.handler.AbstractHandler;
import org.codehaus.xfire.handler.Phase;

import eu.unicore.security.SecurityTokens;
import eu.unicore.security.SignatureStatus;
import eu.unicore.security.util.Log;
import eu.unicore.security.xfireutil.AuthInHandler;
import eu.unicore.security.xfireutil.DSigParseInHandler;
import eu.unicore.security.xfireutil.DSigSecurityInHandler;


/**
 * For interop, it might be necessary to switch off signature checks for certain
 * services (e.g. BES-related services). This can be achieved by adding this 
 * security handler to the service in question. The handler basically claims
 * that the message was signed correctly, independently of the real outcome
 * of the signature check.
 * @see SecurityTokens
 * @author B. Demuth
 */
public class SkipSignatureCheckInHandler extends AbstractHandler
{
	private static Logger logger = Log.getLogger(Log.SECURITY + ".dsig",
			SkipSignatureCheckInHandler.class);
	
	public SkipSignatureCheckInHandler()
	{
		setPhase(Phase.POLICY);
		after(DSigParseInHandler.class.getName());
		after(DSigSecurityInHandler.class.getName());
		after(AuthInHandler.class.getName());
	}
	
	public void invoke(MessageContext ctx) throws Exception
	{
		SecurityTokens securityTokens = (SecurityTokens) ctx.getProperty(
				SecurityTokens.KEY);
		if (securityTokens == null)
		{
			logger.error("No security context found. You should add " + 
					AuthInHandler.class.getName() + " handler.");
			return;
		}
		
		securityTokens.setMessageSignatureStatus(SignatureStatus.OK);
		String soapAction=(String)securityTokens.getContext().get(SecurityTokens.CTX_SOAP_ACTION);
		if(logger.isDebugEnabled()) 
		{
			logger.debug("Signature verification skipped for action "+soapAction);
		}
	}
	
	

}



