package de.fzj.unicore.uas.dsms;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.security.auth.x500.X500Principal;

import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.security.IAttributeSource;
import eu.unicore.security.AuthorisationException;
import eu.unicore.security.SecurityTokens;
import eu.unicore.security.SubjectAttributesHolder;

public class MockAttributeSource implements IAttributeSource {

	private static final Pattern PATTERN = Pattern.compile("cn=([^,]+),");

	public MockAttributeSource() {
	}

	public void init(Kernel k, String name) throws Exception {
	}

	@Override
	public String[] getAcceptedVOs() {
		return new String[0];
	}

	/**
	 * gets attributes based on user's DN
	 * 
	 * @throws AuthorisationException
	 */
	public SubjectAttributesHolder getAttributes(SecurityTokens tokens, SubjectAttributesHolder unused)
			throws IOException {
		String name = tokens.getEffectiveUserName().getName();
		String dn = new X500Principal(name).getName(X500Principal.CANONICAL);

		Matcher m = PATTERN.matcher(dn);
		String login = null;
		if (m.find()) {
			login = m.group(1);
		}
		if (login == null || login.length() == 0) {
			login = "nobody";
		}

		Map<String, String[]> ret = new HashMap<String, String[]>();
		ret.put(IAttributeSource.ATTRIBUTE_XLOGIN, new String[] { login });
		ret.put(IAttributeSource.ATTRIBUTE_ROLE, new String[] { "user" });

		SubjectAttributesHolder holder = new SubjectAttributesHolder(ret);
		unused.addAllOverwritting(holder);
		return unused;
	}

	public String getStatusDescription() {
		return "OK";
	}

	public String getName() {
		return "Mock Attribute Source";
	}

}
