package de.fzj.unicore.uas.dsms;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.unigrids.services.atomic.types.GridFileType;

import pl.edu.icm.dsms.catalogue.CatalogueClient;
import pl.edu.icm.dsms.catalogue.db.InitDB;
import pl.edu.icm.dsms.catalogue.exception.FileNotExistsFault;
import de.fzj.unicore.uas.client.StorageClient;

public class TestDsmsAutoSync extends AbstractTestBase {
	private StorageClient dsms, sms;

	private String dataMap = "src/test/resources/dcatalogue/datamap.properties";

	@BeforeClass
	public static void startKernel() {
		try {
			System.out.println("Starting kernel");
			AbstractTestBase.startKernel();
			kernel.setProperty("uas.dsms.sync1", "true");
			kernel.setProperty("uas.dsms.sync2", "true");
		} catch (Exception e) {
			e.printStackTrace();
			fail("Failed to start kernel");
		}
	}

	@AfterClass
	public static void shutdownKernel() {
		try {
			System.out.println("Shutting down kernel");
			AbstractTestBase.shutdownKernel();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Failed to shutdown kernel");
		}
	}

	public TestDsmsAutoSync() throws Exception {
		super();
	}

	@Before
	public void setUp() {
		try {
			InitDB.initDB(dataMap);
			createSms();
			createCatalogue();
			createDsms();

			dsms = getDsmsClient();
			sms = getSmsClient();
			sms.createDirectory(".dsms");
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unable setup test");
		}
	}

	@After
	public void tearDown() {
		try {
			dsms = null;
			sms = null;

			destroyDsms();
			destroyCatalogue();
			destroySms();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unable to tear down");
		}
	}

	@Test
	// unexpected new file on sms
	public void testAutoSync1() throws IOException, Exception {
		byte body[] = "test file body".getBytes();
		dsms.getImport("test1").write(body);

		sms.getImport(".dsms/test3").write(body);
		dsms.listProperties("test3");
		byte recv[] = dsms.getExport("test3").read(0, 1, body.length, 0);
		assertTrue(new String(body).equals(new String(recv)));
		dsms.delete("test3");
	}

	@Test
	// remove dsms file from sms
	public void testAutoSync2() throws IOException, Exception {
		byte body[] = "test file body".getBytes();
		dsms.getImport("test").write(body);
		sms.delete(".dsms/test");
		boolean exceptionOccured = false;
		try {
			dsms.listProperties("test");
		} catch (FileNotFoundException e) {
			exceptionOccured = true;
		}
		assertTrue(exceptionOccured);

		CatalogueClient cat = getCatalogueClient();
		exceptionOccured = false;
		try {
			cat.lookup("test", null);
		} catch (FileNotExistsFault e) {
			exceptionOccured = true;
		}
		assertTrue(exceptionOccured);
	}

	@Test
	// unexpected new directory on sms
	public void testAutoSync1Dir() throws Exception {
		byte body[] = "test file body".getBytes();
		dsms.getImport("test1").write(body);

		sms.createDirectory(".dsms/newDir");
		GridFileType file = dsms.listProperties("newDir");
		assertTrue(file.getIsDirectory());
		dsms.delete("newDir");
	}
}
