package de.fzj.unicore.uas.dsms;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import pl.edu.icm.dsms.catalogue.db.InitDB;

import de.fzj.unicore.uas.client.MetadataClient;
import de.fzj.unicore.uas.client.RByteIOClient;
import de.fzj.unicore.uas.client.StorageClient;

@Ignore
public class TestDsmsMetadata extends AbstractTestBase {

	protected StorageClient dsms, sms;

	private String dataMap = "src/test/resources/dcatalogue/datamap.properties";

	@BeforeClass
	public static void startKernel() {
		try {
			System.out.println("Starting kernel");
			AbstractTestBase.startKernel();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Failed to start kernel");
		}
	}

	@AfterClass
	public static void shutdownKernel() {
		try {
			System.out.println("Shutting down kernel");
			AbstractTestBase.shutdownKernel();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Failed to shutdown kernel");
		}
	}

	public TestDsmsMetadata() throws Exception {
		super();
	}

	@Before
	public void setUp() throws Exception {
		cleanStorageDirs();
		InitDB.initDB(dataMap);
		createSms();
		createCatalogue();
		createDsms();

		dsms = getDsmsClient();
		sms = getSmsClient();
		sms.createDirectory(".dsms");
	}

	@After
	public void tearDown() {
		try {
			sms = null;
			dsms = null;

			destroyDsms();
			destroyCatalogue();
			destroySms();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unable to tear down");
		}
	}

	@Test
	public void testMetadata() throws Exception {
		RByteIOClient client = dsms.getImport("/foo");
		client.append("test data".getBytes());

		// get metadata client
		MetadataClient mc = dsms.getMetadataClient();
		MetadataClient smsMc = sms.getMetadataClient();
		assertTrue(mc != null);

		// fill some metadata and see we can retrieve them
		Map<String, String> metadata = new HashMap<String, String>();
		metadata.put("ham", "spam");
		metadata.put("test", "123");
		mc.createMetadata("/foo", metadata);

		// get
		Map<String, String> map = mc.getMetadata("/foo");
		assertEquals("spam", map.get("ham"));
		
		// get from sms
		map = smsMc.getMetadata("/.dsms/foo");
		assertEquals("spam", map.get("ham"));

		// update
		map.put("newkey", "newvalue");
		mc.updateMetadata("/foo", map);
		map = mc.getMetadata("/foo");
		assertEquals("newvalue", map.get("newkey"));

		// delete
		mc.deleteMetadata("/foo");
		map = mc.getMetadata("/foo");
		assertEquals(0, map.size());

		// create
		map.put("test", "test-value");
		mc.createMetadata("/foo", map);
		map = mc.getMetadata("/foo");
		assertTrue(0 < map.size());
		assertEquals("test-value", map.get("test"));

		// search
		Collection<String> results = mc.search("foo", false);
		assertEquals(1, results.size());
		assertTrue(results.contains("foo"));

		// extraction
		mc.startMetadataExtraction("/", 4321);
	}
}
