package de.fzj.unicore.uas.dsms;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import pl.edu.icm.dsms.catalogue.db.InitDB;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.StorageFactoryClient;
import de.fzj.unicore.wsrflite.xmlbeans.BaseFault;

public class TestDsmsMultifileSync extends AbstractTestBase {	
	private StorageClient dsms, sms1, sms2, sms3;

	private String dataMap = "src/test/resources/dcatalogue/datamap.properties";

	@BeforeClass
	public static void startKernel() {
		try {
			System.out.println("Starting kernel");
			AbstractTestBase.startKernel();
			kernel.setProperty("uas.dsms.sync1", "true");
			kernel.setProperty("uas.dsms.sync2", "true");
			kernel.setProperty("catalogue.DEFAULT.algorithm", "pl.edu.icm.dsms.catalogue.finder.algorithm.RoundRobin");
		} catch (Exception e) {
			e.printStackTrace();
			fail("Failed to start kernel");
		}
	}

	@AfterClass
	public static void shutdownKernel() {
		try {
			System.out.println("Shutting down kernel");
			AbstractTestBase.shutdownKernel();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Failed to shutdown kernel");
		}
	}

	public TestDsmsMultifileSync() throws Exception {
		super();
	}

	@Before
	public void setUp() {
		try {
			InitDB.initDB(dataMap);
			createFactory();
			StorageFactoryClient factory = getFactoryClient();
			sms1 = factory.createSMS();
			sms2 = factory.createSMS();
			sms3 = factory.createSMS();
			kernel.setProperty("catalogue.DEFAULT.smses", sms1.getEPR().getAddress().getStringValue() + " "
					+ sms2.getEPR().getAddress().getStringValue() + " "
					+ sms3.getEPR().getAddress().getStringValue());
			kernel.setProperty("catalogue.DEFAULT.factories", "");
			sms1.createDirectory(".dsms");
			sms2.createDirectory(".dsms");
			sms3.createDirectory(".dsms");
			
			createCatalogue();
			createDsms();

			dsms = getDsmsClient();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unable setup test");
		}
	}

	@After
	public void tearDown() {
		try {
			dsms = null;
			sms1 = sms2 = sms3 = null;

			destroyDsms();
			destroyCatalogue();
			destroyFactory();
		} catch (Exception e) {
			e.printStackTrace();
			fail("Unable to tear down");
		}
	}

	@Test
	// unexpected new file on 3 smses
	public void testAutoSync1() throws IOException, Exception {
		byte body[] = "test file body".getBytes();
		dsms.getImport("warm-up1").write(body);
		dsms.getImport("warm-up2").write(body);
		dsms.getImport("warm-up3").write(body);

		sms1.getImport(".dsms/test3").write(body);
		sms2.getImport(".dsms/test3").write(body);
		sms3.getImport(".dsms/test3").write(body);
		
		boolean exceptionOccured = false;
		try {
			dsms.listProperties("test3");
		} catch(BaseFault e) {
			exceptionOccured = true;
		}
		assertTrue("Exception didn't occured", exceptionOccured);
	}
	
	@Test
	// unexpected new file appeared on second storage
	public void newFileAppeared() throws IOException, Exception {
		byte body[] = "test file body".getBytes();
		dsms.getImport("warm-up1").write(body);
		dsms.getImport("warm-up2").write(body);
		dsms.getImport("warm-up3").write(body);
		
		sms1.getImport(".dsms/test").write(body);
		sms2.getImport(".dsms/test").write(body);
		sms3.getImport(".dsms/test").write(body);
		
		boolean exceptionOccured = false;
		try {
			dsms.listProperties("test");
		} catch(BaseFault e) {
			exceptionOccured = true;
		}
		assertTrue("Exception didn't occured", exceptionOccured);
	}
	
	@Test
	// unexpected new directory appeared on second storage
	public void newDirAppeared() throws IOException, Exception {
		byte body[] = "test file body".getBytes();
		dsms.getImport("warm-up1").write(body);
		dsms.getImport("warm-up2").write(body);
		dsms.getImport("warm-up3").write(body);
		
		sms1.createDirectory(".dsms/test");
		sms2.getImport(".dsms/test").write(body);
		sms3.createDirectory(".dsms/test");
		
		boolean exceptionOccured = false;
		try {
			dsms.listProperties("test");
		} catch(BaseFault e) {
			exceptionOccured = true;
		}
		assertTrue("Exception didn't occured", exceptionOccured);
	}

}
