DROP TABLE IF EXISTS ldas, owners_files, files, dsmses, smses, owners;
DROP INDEX IF EXISTS d_identifier;
DROP INDEX IF EXISTS s_factory_epr;
DROP INDEX IF EXISTS f_parent_id;
DROP INDEX IF EXISTS f_path;
DROP INDEX IF EXISTS f_path_dir;
DROP INDEX IF EXISTS o_dn;

CREATE TABLE owners (
  id INT IDENTITY NOT NULL,
  owner_dn VARCHAR(512) NULL,
  PRIMARY KEY (id)
);
CREATE UNIQUE INDEX o_dn ON owners (owner_dn);

CREATE TABLE dsmses (
  id INT IDENTITY NOT NULL,
  identifier VARCHAR(64) NOT NULL,
  owner_id INT NULL,
  root VARCHAR(1024) NOT NULL,
  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES owners (id) ON DELETE CASCADE
);
CREATE UNIQUE INDEX d_identifier ON dsmses (identifier);

CREATE TABLE smses (
  id INT IDENTITY NOT NULL,
  dsms_id INT NOT NULL,
  sms_epr VARCHAR(512) NULL,
  factory_epr VARCHAR(512) NULL,
  tsf_epr VARCHAR(512) NULL,
  PRIMARY KEY (id),
  FOREIGN KEY (dsms_id) REFERENCES dsmses (id) ON DELETE CASCADE
);
CREATE INDEX s_factory_epr ON smses (factory_epr);

CREATE TABLE files (
  id INT IDENTITY NOT NULL,
  filename VARCHAR(128) NOT NULL,
  full_path VARCHAR(1024) NOT NULL,
  full_path_md5 VARCHAR(32) NOT NULL,
  parent_id INT NOT NULL,
  directory TINYINT NOT NULL,
  dsms_id INT NOT NULL,
  PRIMARY KEY (id),
  FOREIGN KEY (dsms_id) REFERENCES dsmses (id) ON DELETE CASCADE
);
CREATE INDEX f_parent_id ON files (parent_id);
CREATE INDEX f_path_dir ON files (directory, dsms_id, full_path_md5);
CREATE INDEX f_path ON files (dsms_id, full_path_md5);

CREATE TABLE ldas (
  id INT IDENTITY NOT NULL,
  file_id INT NOT NULL,
  sms_id INT NOT NULL,
  filename VARCHAR(512) NOT NULL,
  directory TINYINT NOT NULL,
  PRIMARY KEY (id),
  FOREIGN KEY (file_id) REFERENCES files (id) ON DELETE CASCADE,
  FOREIGN KEY (sms_id) REFERENCES smses (id) ON DELETE CASCADE
);

CREATE TABLE owners_files (
  owner_id INT NOT NULL,
  file_id INT NOT NULL,
  PRIMARY KEY (owner_id, file_id),
  FOREIGN KEY (owner_id) REFERENCES owners (id) ON DELETE CASCADE,
  FOREIGN KEY (file_id) REFERENCES files (id) ON DELETE CASCADE
);
